*==============================================================================
* Class:					OfficeHandler
* Based On:				Session
* Purpose:					Implements Office server events
* Author:					Tamar E. Granor
* Copyright:				(c) 2003 Tamar E. Granor, Ph.D.
* Last revision:				3/28/2003
* Include file:				None
*==============================================================================
* Refactored 5/7/2003 due to bug preventing both Excel and PowerPoint events from being
* implemented in the same class. This version uses separate classes to implement each
* server's events. The parent class maintains a collection of instantiated event handlers 
* and has a method, GetHandler, to return a reference to a handler of a specified type.
***************************************************************************************************************************
DEFINE CLASS OfficeHandler AS session OLEPUBLIC

	PROTECTED cLogTable, cLogPath, cLogFullPath
	cLogTable = "OfficeLogEventHandler.DBF"
  	colHandlers = .null.	&& collection of actual handlers

	PROCEDURE Init
	
	This.cLogPath = SYS(2023)
	This.cLogFullPath = FORCEPATH(This.cLogTable, ;
    	                          This.cLogPath)

	* Make sure the log table exists
	IF NOT FILE(This.cLogFullPath)
    	CREATE TABLE (This.cLogFullPath) ;
          (cDocument C(60), cAction C(12), tModified T)
        CLOSE TABLES 
	ENDIF
	
	SET EXCLUSIVE OFF 
	RETURN 

	PROCEDURE GetHandler
	* Return an object reference to a handler of the specified type.
	* Create it, if necessary.
	
	LPARAMETERS cHandlerType
	
	ASSERT VARTYPE(cHandlerType)="C" AND NOT EMPTY(cHandlerType) ;
	 	MESSAGE "GetHandler: Must include handler type"
	 	
	 IF VARTYPE(cHandlerType) <> "C" OR EMPTY(cHandlerType)
	 	ERROR 11
	 	RETURN .null.
	 ENDIF
	 
	 LOCAL oHandler, oEventHandler
	 
	 oEventHandler = .null.
	 IF NOT ISNULL(This.colHandlers)
		 FOR EACH oHandler IN This.colHandlers
		 	IF UPPER(oHandler.cType) == UPPER(cHandlerType)
		 		oEventHandler = oHandler.oEventHandler
		 		EXIT
		 	ENDIF
		ENDFOR
	ENDIF
		
	IF ISNULL(oEventHandler)
		* Try to instantiate
		TRY
			oEventHandler = NEWOBJECT(cHandlerType, "OfficeEventHandler.PRG")
			oHandler = CREATEOBJECT("empty")
			ADDPROPERTY(oHandler,"oEventHandler",m.oEventHandler)
			ADDPROPERTY(oHandler,"cType",m.cHandlerType)
			IF ISNULL(This.colHandlers)
				This.colHandlers = CREATEOBJECT("Collection")
			ENDIF 
			This.colHandlers.Add( oHandler )
		CATCH
			ERROR "Can't create event handler of type " + m.cHandlerType
		ENDTRY
		
	ENDIF
	
	RETURN oEventHandler

	
ENDDEFINE

DEFINE CLASS WordHandler AS OfficeHandler  OLEPUBLIC

    * Word
    IMPLEMENTS applicationevents2 in {00020905-0000-0000-C000-000000000046}#8.1

	*****
	* Begin Word events
	*****
	PROCEDURE ApplicationEvents2_DocumentOpen(Doc AS VARIANT) AS VOID

	* Log open
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Doc.FullName, "Opened", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE ApplicationEvents2_DocumentBeforeClose(Doc AS VARIANT, Cancel AS LOGICAL) AS VOID

	* Log close
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Doc.FullName, "Closed", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE ApplicationEvents2_DocumentBeforeSave(Doc AS VARIANT, SaveAsUI AS LOGICAL, Cancel AS LOGICAL) AS VOID

	* Log save
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Doc.FullName, "Saved", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE ApplicationEvents2_NewDocument(Doc AS VARIANT) AS VOID

	* Log create
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Doc.FullName, "Created", DATETIME())

	RETURN .t.

	ENDPROC

	*****
	* Not using these events of Word 
	*****
	PROCEDURE ApplicationEvents2_DocumentBeforePrint(Doc AS VARIANT, Cancel AS LOGICAL) AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents2_Quit() AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents2_DocumentChange() AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents2_WindowActivate(Doc AS VARIANT, Wn AS VARIANT) AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents2_WindowDeactivate(Doc AS VARIANT, Wn AS VARIANT) AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents2_WindowSelectionChange(Sel AS VARIANT) AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents2_WindowBeforeRightClick(Sel AS VARIANT, Cancel AS LOGICAL) AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents2_WindowBeforeDoubleClick(Sel AS VARIANT, Cancel AS LOGICAL) AS VOID
		* add user code here
	ENDPROC
	
ENDDEFINE

DEFINE CLASS ExcelHandler AS OfficeHandler OLEPUBLIC

    * Excel
    IMPLEMENTS AppEvents IN "Excel.Application"

	*****
	* Begin Excel events
	*****

	PROCEDURE AppEvents_NewWorkbook(Wb AS VARIANT) AS VOID

	* Log create
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Wb.FullName, "Created", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE AppEvents_WorkbookOpen(Wb AS VARIANT) AS VOID

	* Log open
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Wb.FullName, "Opened", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE AppEvents_WorkbookBeforeClose(Wb AS VARIANT, Cancel AS LOGICAL) AS VOID

	* Log close
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Wb.FullName, "Closed", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE AppEvents_WorkbookBeforeSave(Wb AS VARIANT, SaveAsUI AS LOGICAL, Cancel AS LOGICAL) AS VOID

	* Log save
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Wb.FullName, "Saved", DATETIME())

	RETURN .t.

	ENDPROC

	*****
	* Not using these events of Excel
	*****
	PROCEDURE AppEvents_WorkbookActivate(Wb AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookDeactivate(Wb AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetSelectionChange(Sh AS VARIANT, Target AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetBeforeDoubleClick(Sh AS VARIANT, Target AS VARIANT, Cancel AS LOGICAL) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetBeforeRightClick(Sh AS VARIANT, Target AS VARIANT, Cancel AS LOGICAL) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetActivate(Sh AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetDeactivate(Sh AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetCalculate(Sh AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetChange(Sh AS VARIANT, Target AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookBeforePrint(Wb AS VARIANT, Cancel AS LOGICAL) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookNewSheet(Wb AS VARIANT, Sh AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookAddinInstall(Wb AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookAddinUninstall(Wb AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WindowResize(Wb AS VARIANT, Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WindowActivate(Wb AS VARIANT, Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WindowDeactivate(Wb AS VARIANT, Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetFollowHyperlink(Sh AS VARIANT, Target AS VARIANT) AS VOID
	* add user code here
	ENDPROC

ENDDEFINE

DEFINE CLASS PPTHandler AS OfficeHandler OLEPUBLIC

    * PowerPoint
    IMPLEMENTS EApplication IN "PowerPoint.Application"

	*****
	* Begin PowerPoint events
	*****
	PROCEDURE EApplication_PresentationClose(Pres AS VARIANT) AS VOID
	* Log close 
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Pres.FullName, "Closed", DATETIME())

	RETURN .t.
	ENDPROC

	PROCEDURE EApplication_PresentationSave(Pres AS VARIANT) AS VOID
	* Log save 
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Pres.FullName, "Saved", DATETIME())

	RETURN .t.
	ENDPROC

	PROCEDURE EApplication_PresentationOpen(Pres AS VARIANT) AS VOID
	* Log open
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Pres.FullName, "Opened", DATETIME())

	RETURN .t.
	ENDPROC

	PROCEDURE EApplication_NewPresentation(Pres AS VARIANT) AS VOID
	* Log creation
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Pres.FullName, "Created", DATETIME())

	RETURN .t.
	ENDPROC

	*****
	* Not using these events of PowerPoint
	*****
	PROCEDURE EApplication_WindowSelectionChange(Sel AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_WindowBeforeRightClick(Sel AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_WindowBeforeDoubleClick(Sel AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_PresentationNewSlide(Sld AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_WindowActivate(Pres AS VARIANT, Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_WindowDeactivate(Pres AS VARIANT, Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_SlideShowBegin(Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_SlideShowNextBuild(Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_SlideShowNextSlide(Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_SlideShowEnd(Pres AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_PresentationPrint(Pres AS VARIANT) AS VOID
	* add user code here
	ENDPROC


ENDDEFINE

